import emod_api.interventions.utils as utils
from emod_api import schema_to_class as s2c

import copy
from typing import List

schema_path=None
old_adhoc_trigger_style=True
###
### Generic
###
def BroadcastEvent(
        camp,
        Event_Trigger: str = 'Births'
    ):
    """
        Wrapper function to create and return a BroadcastEvent intervention.

        Args:
            camp: emod_api.campaign object with schema_path set.
            Event_Trigger: A valid trigger/event/signal.

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            BroadastEvent intervention ready to be added to a campaign.
    """
    if Event_Trigger is None or Event_Trigger == "":
        raise ValueError( "BroadcastEvent called with an empty Event_Trigger. Please specify a string." )

    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )
    intervention = s2c.get_class_with_defaults( "BroadcastEvent", schema_path )
    intervention.Broadcast_Event = camp.get_send_trigger( Event_Trigger, old=old_adhoc_trigger_style )
    return intervention


def BroadcastEventToOtherNodes(
        camp,
        Event_Trigger,
        Node_Selection_Type="DISTANCE_ONLY",
        Max_Distance_To_Other_Nodes_Km=-1,
        Include_My_Node=1
    ):
    """
        Wrapper function to create and return a BroadcastEventToOtherNodes intervention. 

        Args:
            camp: emod_api.campaign object with schema_path set.
            Event_Trigger: A valid trigger/event/signal.
            Node_Selection_Type: TBD.
            Max_Distance_To_Other_Nodes_Km: TBD.
            Include_My_Node: TBD.

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            BroadastEvent intervention ready to be added to a campaign.
    """
    if Event_Trigger is None or Event_Trigger == "":
        raise ValueError( "BroadcastEventToOtherNodes called with an empty Event_Trigger. Please specify a string." )

    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )
    intervention = s2c.get_class_with_defaults( "BroadcastEventToOtherNodes", schema_path )
    intervention.Event_Trigger = camp.get_send_trigger( Event_Trigger, old=old_adhoc_trigger_style )
    intervention.Node_Selection_Type = Node_Selection_Type 
    if Max_Distance_To_Other_Nodes_Km != -1:
        intervention.Max_Distance_To_Other_Nodes_Km = Max_Distance_To_Other_Nodes_Km 
    intervention.Include_My_Node = Include_My_Node 
    return intervention

def MultiInterventionDistributor(
        camp,
        Intervention_List
    ):
    """
        Wrapper function to create and return a MultiInterventionDistributor intervention. 

        Args:
            camp: emod_api.campaign object with schema_path set.
            Intervention_List: List of 1 or more valid intervention dictionaries to be
            distributed together. 

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            MultiInterventionDistributor intervention ready to be added to a campaign.
    """
   
    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )
    if Intervention_List is None or type(Intervention_List) is not list or len( Intervention_List ) == 0:
        raise ValueError( "Intervention_List is empty or None or not a list." )
    intervention = s2c.get_class_with_defaults( "MultiInterventionDistributor", schema_path )
    intervention.Intervention_List = copy.deepcopy( Intervention_List )
    if "Intervention_Name" in Intervention_List[0]:
        intervention.Intervention_Name = Intervention_List[0].Intervention_Name
    else:
        intervention.Intervention_Name = "MultiInterventionDistributor"
    return intervention


def DelayedIntervention(
        camp,
        Configs,
        Delay_Dict=None
    ):
    """
        Wrapper function to create and return a DelayedIntervention intervention. 

        Args:
            camp: emod_api.campaign object with schema_path set.  
            Config: Valid intervention config.
            Delay_Dict: Dictionary of 1 or 2 params that are the literal Delay_Distribution
            parameters, but without the distribution, which is inferred. E.g., 
            { "Delay_Period_Exponential": 5 }

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            DelayedIntervention intervention ready to be added to a campaign.
    """
    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )

    if Configs is None or type( Configs ) is not list or len( Configs ) == 0:
        raise ValueError( "Configs is empty or None or not a list." )

    intervention = s2c.get_class_with_defaults( "DelayedIntervention", schema_path )
    intervention.Actual_IndividualIntervention_Configs = copy.deepcopy( Configs )
    if Delay_Dict is None:
        Delay_Dict = { "Delay_Period_Constant": 0 }
    for param in Delay_Dict:
        setattr( intervention, param, Delay_Dict[ param ] )
    return intervention


def HSB(
        camp,
        Event_Or_Config="Event",
        Config=None,
        Event="NoTrigger",
        Tendency=1.0,
        Single_Use=True,
        Name="HSB"
    ):
    """
        Wrapper function to create and return a HealthSeekingBehaviour intervention. 

        Args:
            camp: emod_api.campaign object with schema_path set.
            Event_Or_Config: "Event" or "Config".
            Config: Complete, valid intervention configuration to be distributed.
            Event: Event/Trigger/Signal to be broadcast, alternative to an intervention.
            Tendency: Daily probability of 'seeking care' aka distributing payload intervention.
            Single_Use: One-and-done, or continuous?
            Name: Intervention Name. Useful if you want to provide uniqueness and not worry about 
            duplicate intervention management.

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            HSB intervention ready to be added to a campaign.
    """
    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )

    intervention = s2c.get_class_with_defaults( "SimpleHealthSeekingBehavior", schema_path )
    intervention.Event_Or_Config = Event_Or_Config
    if intervention.Event_Or_Config == "Event":
        intervention.Actual_IndividualIntervention_Event = camp.get_send_trigger( Event, old=old_adhoc_trigger_style )
    else:
        if Config is None:
            raise ValueError( "You specified 'Config' but no actual Config." )
        intervention.Actual_IndividualIntervention_Config = Config
    intervention.Tendency = Tendency 
    intervention.Intervention_Name = Name 
    intervention.Single_Use = Single_Use 
    return intervention


def NLHTI(
        camp,
        Triggers,
        Interventions,
        Property_Restrictions=None,
        Demographic_Coverage=1.0,
        Target_Age_Min=0,
        Target_Age_Max=125*365,
        Target_Gender="All",
        Target_Residents_Only=1,
        Duration=-1,
        Blackout_Event_Trigger=None,
        Blackout_Period=None,
        Blackout_On_First_Occurrence=None,
        Disqualifying_Properties=None,
    ):
    """
        Wrapper function to create and return a NodeLevelHealthTriggeredIntervention intervention. 

        Args:
            camp: emod_api.campaign object with schema_path set.
            Triggers: List of Triggers/Events/Signals
            Interventions: List of interventions to distrbute when signal is heard.
            Property_Restrictions: Individual Properties that an agent must have to qualify for intervention.
            Demographic_Coverage: Percentage of individuals to receive intervention.
            Target_Age_Min: Minimum age (in years).
            Target_Age_Max: Maximum age (in years).
            Target_Gender: All, Male, or Female.
            Target_Residents_Only: Not used.
            Duration: How long this listen-and-distribute should last.
            Blackout_Event_Trigger: Not used.
            Blackout_Period: Not used.
            Blackout_On_First_Occurrence: Not used.
            Disqualifying_Properties: Not used.

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            NLHTI intervention ready to be added to a campaign.
    """
    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )

    intervention = s2c.get_class_with_defaults( "NodeLevelHealthTriggeredIV", schema_path )
    intervention.Trigger_Condition_List = [ camp.get_recv_trigger( trigger, old=old_adhoc_trigger_style ) for trigger in Triggers ]
    if Target_Age_Min > 0 or Target_Age_Max < 125*365:
        intervention.Target_Age_Min = Target_Age_Min 
        intervention.Target_Age_Max = Target_Age_Max
    if Target_Gender != "All":
        intervention.Target_Gender = Target_Gender 
        intervention.Target_Demographic = "ExplicitAgeRangesAndGender"
    intervention.Demographic_Coverage = Demographic_Coverage 
    intervention.Duration = Duration
    intervention.Blackout_Event_Trigger = Blackout_Event_Trigger
    intervention.Blackout_Period = Blackout_Period
    intervention.Blackout_On_First_Occurrence = Blackout_On_First_Occurrence
    prs = utils._convert_prs( Property_Restrictions )
    if len(prs)>0 and type(prs[0]) is dict:
        intervention.Property_Restrictions_Within_Node = prs
        intervention.pop( "Property_Restrictions" )
    else:
        intervention.Property_Restrictions = prs
        intervention.pop( "Property_Restrictions_Within_Node" )
    if len(Interventions) == 1:
        intervention.Actual_IndividualIntervention_Config = Interventions[0]
    else:
        intervention.Actual_IndividualIntervention_Config = MultiInterventionDistributor( camp, Interventions )

    return intervention

 
def PropertyValueChanger(
        camp,
        Target_Property_Key,
        Target_Property_Value,
        Daily_Probability=1.0,
        Maximum_Duration=1,
        Revert=-1,
        Intervention_Name="",
        Event_Trigger_Distributed="",
        Event_Trigger_Expired=""
    ):
    """
        Wrapper function to create and return a PropertyValueChanger intervention. 

        Args:
            camp: emod_api.campaign object with schema_path set. 
            Target_Property_Key. The key part of the new key-value pair of the IP.
            Target_Property_Value. The value part of the new key-value pair of the IP.
            New_Property_Value.. Optional IP key:value part to be set, common to all interventions.
            Daily_Probability. The daily probability that an individual will move to the Target_Property_Value.
            Maximum_Duration. The maximum amount of time individuals have to move to a new group. This timing works in conjunction with Daily_Probability.
            Revert.  The number of days before an individual moves back to their original group.
            Intervention_Name. Optional Intervention_Name. Useful if managing a replacement policy.
            Event_Trigger_Distributed. Optional broadcast trigger to be published when PVC is distributed.
            Event_Trigger_Expired. Optional broadcast trigger to be published when PVC is expired.

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            PropertyValueChanger intervention ready to be added to a campaign.
    """
    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )

    intervention = s2c.get_class_with_defaults( "PropertyValueChanger", schema_path )
    intervention.Target_Property_Key = Target_Property_Key
    intervention.Target_Property_Value = Target_Property_Value
    intervention.Daily_Probability = Daily_Probability 
    intervention.Maximum_Duration=Maximum_Duration
    if Revert != -1:
        intervention.Revert=Revert
    if len(Intervention_Name)>0:
        intervention.Intervention_Name=Intervention_Name
    if len(Event_Trigger_Distributed)>0:
        intervention.Event_Trigger_Distributed=Event_Trigger_Distributed
    if len(Event_Trigger_Expired)>0:
        intervention.Event_Trigger_Expired=Event_Trigger_Expired

    return intervention


def ScheduledCampaignEvent(
        camp,
        Start_Day: int,
        Node_Ids = None,
        Nodeset_Config = None,
        Number_Repetitions: int = 1,
        Timesteps_Between_Repetitions: int = -1,
        Event_Name: str = "Scheduled_Campaign_Event", # not set, doesn't exist in schema
        Property_Restrictions = None,
        Demographic_Coverage:float = 1.0,
        Target_Age_Min=0,
        Target_Age_Max=125*365,
        Target_Gender:str = "All",
        Intervention_List=None):

    """
        Wrapper function to create and return a ScheduledCampaignEvent intervention. 
        The alternative to a ScheduledCampaignEvent is a TriggeredCampaignEvent.

        Args:
            camp: emod_api.campaign object with schema_path set.
            Start_Day: When to start.
            Event_Name: Name for overall campaign event, of no functional meaning. Not in schema and not yet used.
            Node_Ids: Nodes to target with this intervenion
            Nodeset_Config: Nodes to target with this intervenion, return from utils.do_nodes().

                .. deprecated:: 2.x
                   Use parameter Node_Ids instead

            Property_Restrictions: Individual Properties a person must have to receive the intervention(s).
            Number_Repetitions: N/A 
            Timesteps_Between_Repetitions: N/A 
            Demographic_Coverage: Percentage of individuals to receive intervention.
            Target_Age_Min: Minimum age (in years).
            Target_Age_Max: Maximum age (in years).
            Target_Gender: All, Male, or Female.
            Intervention_List: List of 1 or more valid intervention dictionaries to be
            distributed together. 

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            ScheduledCampaignEvent intervention ready to be added to a campaign.
    """
    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )

    # Not checking Intervention_list because MultiInterventionDistributor checks

    event = s2c.get_class_with_defaults( "CampaignEvent", schema_path )
    coordinator = s2c.get_class_with_defaults( "StandardEventCoordinator", schema_path )
    coordinator.Demographic_Coverage = Demographic_Coverage
    coordinator.Number_Repetitions=Number_Repetitions
    coordinator.Timesteps_Between_Repetitions=Timesteps_Between_Repetitions

    # Second, hook them up
    event.Event_Coordinator_Config = coordinator
    event.Start_Day = float(Start_Day)

    if Nodeset_Config is not None and Node_Ids is not None:
        raise AssertionError("Node_Ids and Nodeset_Config are set. Please set only one.")

    if Nodeset_Config is not None:
        event.Nodeset_Config = Nodeset_Config
    else:
        # The real default is an empty list
        if Node_Ids is None:
            Node_Ids = []
        node_ids = Node_Ids
        event.Nodeset_Config = utils.do_nodes(camp.schema_path, node_ids)

    coordinator.Intervention_Config = MultiInterventionDistributor( camp, Intervention_List ) 
    prs = utils._convert_prs( Property_Restrictions )
    if len(prs)>0 and type(prs[0]) is dict:
        coordinator.Property_Restrictions_Within_Node = prs
        coordinator.pop( "Property_Restrictions" )
    else:
        coordinator.Property_Restrictions = prs
        coordinator.pop( "Property_Restrictions_Within_Node" )
    
    coordinator.Demographic_Coverage = Demographic_Coverage

    if Target_Age_Min > 0 or Target_Age_Max < 125*365:
        coordinator.Target_Age_Min = Target_Age_Min

    if Target_Age_Max is not None and Target_Age_Max < 125*365:  # overwrite default from schema
        coordinator.Target_Age_Max = Target_Age_Max

    if Target_Gender != "All":
        coordinator.Target_Gender = Target_Gender 
        coordinator.Target_Demographic = "ExplicitAgeRangesAndGender"
    
    return event


def TriggeredCampaignEvent(
        camp,
        Start_Day: int,
        Event_Name: str,
        Triggers: List[str],
        Intervention_List: List[dict],
        Node_Ids=None,
        Nodeset_Config=None,
        Node_Property_Restrictions=None,
        Property_Restrictions=None,
        Number_Repetitions: int = 1,
        Timesteps_Between_Repetitions: int = -1,
        Demographic_Coverage: float=1.0,
        Target_Age_Min=0,
        Target_Age_Max=125*365,
        Target_Gender: str ="All",
        Target_Residents_Only=1,
        Duration=-1,
        Blackout_Event_Trigger: str=None,
        Blackout_Period=0,
        Blackout_On_First_Occurrence=0,
        Disqualifying_Properties=None,
        Delay=None
    ):
    """
        Wrapper function to create and return a TriggeredCampaignEvent intervention. 
        The alternative to a TriggeredCampaignEvent is a ScheduledCampaignEvent.

        Args:
            camp: emod_api.campaign object with schema_path set.
            Start_Day: When to start.
            Event_Name: Name for overall campaign event, of no functional meaning. Not in schema and not yet used.
            Node_Ids: Nodes to target with this intervenion
            Nodeset_Config: Nodes to target with this intervenion, return from utils.do_nodes().

                .. deprecated:: 2.x
                   Use parameter Node_Ids instead

            Triggers: List of triggers/events/signals to listen to in order to trigger distribution.
            Intervention_List: List of 1 or more valid intervention dictionaries to be
            distributed together. 
            Node_Property_Restrictions: N/A.
            Property_Restrictions: Individual Properties a person must have to receive the intervention(s).
            Demographic_Coverage: Percentage of individuals to receive intervention.
            Target_Age_Min: Minimum age (in years).
            Target_Age_Max: Maximum age (in years).
            Target_Gender: All, Male, or Female.
            Target_Residents_Only: Not used.
            Duration: How long this listen-and-distribute should last.
            Blackout_Event_Trigger: Not used.
            Blackout_Period: Not used.
            Blackout_On_First_Occurrence: Not used.
            Disqualifying_Properties: Not used.
            delay: Optional delay between trigger and actual distribution.

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            TriggeredCampaignEvent intervention ready to be added to a campaign.
    """
    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )

    if Node_Property_Restrictions is None:
        Node_Property_Restrictions=[]
    if Property_Restrictions is None:
        Property_Restrictions=[]

    event = s2c.get_class_with_defaults( "CampaignEvent", schema_path )
    event.Start_Day = float(Start_Day)

    if Nodeset_Config is not None and Node_Ids is not None:
        raise AssertionError("Node_Ids and Nodeset_Config are set. Please use only one.")

    if Nodeset_Config is not None:
        event.Nodeset_Config = Nodeset_Config
    else:
        # The real default is an empty list.
        if Node_Ids is None:
            Node_Ids = []
        node_ids = Node_Ids
        event.Nodeset_Config = utils.do_nodes(camp.schema_path, node_ids)

    coordinator = s2c.get_class_with_defaults( "StandardEventCoordinator", schema_path )
    coordinator.Number_Repetitions = Number_Repetitions
    coordinator.Timesteps_Between_Repetitions = Timesteps_Between_Repetitions

    if Delay is not None:
        Intervention_List = [ DelayedIntervention( camp, Intervention_List, Delay_Dict = { "Delay_Period_Constant": Delay } ) ]

    intervention = NLHTI(
        camp,
        Triggers,
        Intervention_List,
        Duration=Duration,
        Property_Restrictions = Property_Restrictions,
        Demographic_Coverage = Demographic_Coverage,
        Target_Age_Min = Target_Age_Min,
        Target_Age_Max = Target_Age_Max,
        Target_Gender = Target_Gender,
        Blackout_Event_Trigger=Blackout_Event_Trigger,
        Blackout_Period=Blackout_Period,
        Blackout_On_First_Occurrence=Blackout_On_First_Occurrence
    )

    event.Event_Coordinator_Config = coordinator
    coordinator.Intervention_Config = intervention
    return event

def StandardDiagnostic(
        camp,
        Base_Sensitivity: float=1.0,
        Base_Specificity: float=1.0,
        Days_To_Diagnosis: float=0.0,
        Event_Trigger_Distributed: str = None,
        Event_Trigger_Expired: str = None,
        Positive_Diagnosis_Intervention = None,
        Positive_Diagnosis_Event: str = "PositiveResult",
        Negative_Diagnosis_Intervention = None,
        Negative_Diagnosis_Event: str = "NegativeResult",
        Treatment_Fraction: float=1.0
    ):
    """
        Wrapper function to create and return a StandardDiagnostic intervention. 

        Args:
            camp: emod_api.campaign object with schema_path set.
            Base_Sensitivity: base sensitivity [0..1]
            Base_Specificity: base specificity [0..1]
            Days_To_Diagnosis: days to diagnosis
            Event_Trigger_Distributed: A trigger that is fired when intervention was distributed
            Event_Trigger_Expired: A trigger that is fired when intervention has expired
            Positive_Diagnosis_Intervention: Intervention that is distributed in case of a positive diagnosis. If set, no events may be configured.
            Positive_Diagnosis_Event: A trigger that is fired in case of a positive diagnosis
            Negative_Diagnosis_Intervention: Intervention that is distributed in case of a Negative diagnosis. If set, no events may be configured. 
                Not used outside of Malaria-Ongoing yet.
            Negative_Diagnosis_Event: A trigger that is fired in case of a Negative diagnosis. Not used outside of Malaria-Ongoing yet.
            Treatment_Fraction: treatment fraction [0..1]

        Returns:
            ReadOnlyDict: Schema-based smart dictionary representing a new 
            MultiInterventionDistributor intervention ready to be added to a campaign.
    """

    global schema_path
    schema_path = ( camp.schema_path if camp is not None else schema_path )

    if Positive_Diagnosis_Intervention is not None and (Event_Trigger_Distributed is not None or Event_Trigger_Expired is not None):
        raise Exception("Events and intervention are configured. Configuration of events and intervention are mutually exclusive.")


    # First, get the objects
    # Only in Malaria-Ongoing the class is called StandardDiagnostic
    try:
        intervention = s2c.get_class_with_defaults("StandardDiagnostic", schema_path)
        if Positive_Diagnosis_Intervention is None:
            intervention.Positive_Diagnosis_Config = BroadcastEvent( camp, Positive_Diagnosis_Event )
        else:
            intervention.Positive_Diagnosis_Config = Positive_Diagnosis_Intervention

        if Negative_Diagnosis_Intervention is None:
            intervention.Negative_Diagnosis_Config = BroadcastEvent( camp, Negative_Diagnosis_Event )
        else:
            intervention.Negative_Diagnosis_Config = Negative_Diagnosis_Intervention
    except ValueError:
        # Non Malaria, use SimpleDiagnostic
        intervention = s2c.get_class_with_defaults("SimpleDiagnostic", schema_path)

        if Positive_Diagnosis_Intervention is None:
            intervention.Positive_Diagnosis_Config = BroadcastEvent( camp, Positive_Diagnosis_Event )
            # The line below works fine instead of the line above but it just makes testing harder.
            #intervention.Positive_Diagnosis_Event = camp.get_send_trigger( Positive_Diagnosis_Event, old=old_adhoc_trigger_style )
            intervention.Event_Trigger_Distributed = Event_Trigger_Distributed
            intervention.Event_Trigger_Expired = Event_Trigger_Expired
        else:
            intervention.Positive_Diagnosis_Config = Positive_Diagnosis_Intervention

    intervention.Base_Sensitivity = Base_Sensitivity
    intervention.Base_Specificity = Base_Specificity
    intervention.Days_To_Diagnosis = Days_To_Diagnosis
    intervention.Treatment_Fraction = Treatment_Fraction

    return intervention


def triggered_campaign_delay_event( camp, start_day, trigger, delay, intervention, ip_targeting=[], coverage=1.0 ):
    """
        Create and return a campaign event that responds to a trigger after a delay with an intervention.

        Args:
            camp: emod_api.campaign object with schema_path set.
            start_day: When to start.
            delay: Dictionary of 1 or 2 params that are the literal Delay_Distribution parameters, 
            but without the distribution, which is inferred. E.g., { "Delay_Period_Exponential": 5 }.
            trigger: E.g., "NewInfection".
            intervention: List of 1 or more valid intervention dictionaries to be distributed together. 
            ip_targeting: Optional Individual Properties required for someone to receive the intervntion(s).

        Returns:
            Campaign event.

    """
    delay_iv = DelayedIntervention( camp, Configs=[intervention], Delay_Dict = delay )
    event = TriggeredCampaignEvent( camp, Start_Day=start_day, Event_Name="triggered_delayed_intervention", Triggers=[ camp.get_recv_trigger( trigger, old=True ) ], Intervention_List=[delay_iv], Property_Restrictions=ip_targeting, Demographic_Coverage=coverage )

    return event


def change_individual_property_at_age( camp, new_ip_key, new_ip_value, change_age_in_days, revert_in_days, ip_targeting_key, ip_targeting_value, coverage=1.0 ):
    """
        Create and return a campaign event that changes a person's Individual Properties once they turns a certain age. 
        e.g., change_individual_property_at_age(cb, 'ForestGoing', 'LovesForest', coverage=0.6, change_age_in_days=15*365, revert=20*365)

        Args:
            camp: emod_api.campaign object with schema_path set.
            new_ip_key: The new IP key.
            new_ip_value: The new IP value.
            change_age_in_days: The age at which the individual transitions (in units of days).
            revert_in_days: How many days they remain with the new property.
            ip_targeting_key: The IP key a person must have to receive this.
            ip_targeting_value: The IP value a person must have to receive this.
            coverage: Optional fraction to limit this to a subset of the target population.

        Returns:
            Campaign event.

    """
    iv = PropertyValueChanger( camp, Target_Property_Key=new_ip_key, Target_Property_Value=new_ip_value, Revert=revert_in_days)
    return triggered_campaign_delay_event( camp, start_day=1, trigger="Births", coverage=coverage, delay={ "Delay_Period_Constant": change_age_in_days }, intervention=iv, ip_targeting={ ip_targeting_key : ip_targeting_value } )
